from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

    lines = long_description.strip()
    lines = lines.split("\n")
    lines = [line.strip() for line in lines]
    lines = [line for line in lines if line]

    package_name = lines[0].lstrip("#").strip()
    print("package_name:", package_name)

    description = lines[1]
    print("description:", description)

setup(
    name=package_name,
    description=description,

    author="Parantapa Bhattacharya",
    author_email="pb+pypi@parantapa.net",

    long_description=long_description,
    long_description_content_type="text/markdown",

    packages=[package_name],

    use_scm_version=True,
    setup_requires=['setuptools_scm'],

    url="http://github.com/parantapa/%s" % package_name,
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)



"""
Subprocess wrapper

Proivdes a subclass of Popen,
that terminates gracefully instead of leaving zombie/orphan subprocesses.
"""

import logging
from subprocess import (Popen, TimeoutExpired)

log = logging.getLogger(__name__)

DEFAULT_WAIT_TIMEOUT = 0
DEFAULT_TERM_TIMEOUT = 30

def terminate_gracefully(process,
                         wait_timeout=DEFAULT_WAIT_TIMEOUT,
                         term_timeout=DEFAULT_TERM_TIMEOUT):
    """
    Terminate the process gracefully.
    """

    try:
        process.wait(timeout=wait_timeout)
    except TimeoutExpired:
        log.info("Terminating %d: %s", process.pid, process.args)
        try:
            process.terminate()
        except ProcessLookupError:
            return

        try:
            process.wait(timeout=term_timeout)
        except TimeoutExpired:
            log.warning("Killing %d: %s", process.pid, process.args)
            try:
                process.kill()
            except ProcessLookupError:
                return

            process.wait()

class PopenW(Popen):
    """
    Popen wrapper that exits gracefully.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        log.info("Starting %d: %s", self.pid, self.args)
        self._log_exited = True

    def is_dead(self):
        """
        Check and log if the process is dead.
        """

        if self.poll() is not None:
            if self._log_exited:
                log.info("Exited %d: %s", self.pid, self.args)
                self._log_exited = False

            return True

        return False

    def terminate_gracefully(self,
                             wait_timeout=DEFAULT_WAIT_TIMEOUT,
                             term_timeout=DEFAULT_TERM_TIMEOUT):
        """
        Terminate the process gracefully.
        """

        if self.is_dead():
            return

        terminate_gracefully(self, wait_timeout, term_timeout)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, value, traceback):
        self.terminate_gracefully()

    def __del__(self):
        self.terminate_gracefully()
